package de.mutschler.rpdl.basicvalue;

public class ValueInt extends BasicValue implements java.io.Serializable
{
	static final long serialVersionUID = 12828067786883851L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------

	/** Here the actual value is stored */
	private int value;
	
	//----------------------------------------------------------------------
	//------- Constructor(s)
	//----------------------------------------------------------------------
	public ValueInt() {this(0);}
	
	public ValueInt (String s) {
		super(s);
		value = Integer.parseInt(s);
	}
	
	public ValueInt (int i) {
		super();
		value = i;
	}
	
	//----------------------------------------------------------------------
	//------- Methods
	//----------------------------------------------------------------------
	
	/** The name of this type. */
	public static final String TYPENAME="INT";
	/** The name of this type. */
	public String getType() {return TYPENAME;}
	
	/** Return the cast operation for casting into the same value type. */
	public int getCastOp() {return BasicValue.TO_INT;}

	/** Get the actual value of this type. */
	public int getValue() {return value;}
	
	/** Get a string-representation of the value. This is used for concatenation. */
	public String getStringValue() {return Integer.toString(value);}

	public void setValue(BasicValue t) throws RpdlConversionException
	{
		if (TYPENAME.equals(t.getType())) {
			value = ((ValueInt)t).getValue();
			return;
		}
		// the superclass can throw the excpeption
		super.setValue(t);
	}
	
	//----------------------------------------------------------------------
	//------- mathematical operations
	//----------------------------------------------------------------------

	/** Perform a calculation. */
	public BasicValue calculate(int op, BasicValue t) throws RpdlConversionException
	{
		String type = ((t==null)?null:t.getType());
		switch(op) {
			case TO_INT:
				return this;
			case TO_NUMBER:
				return new ValueNumber((float)value);
			case TO_DICE: {
					ValueDice d = new ValueDice();
					d.addDice(1, value);
					return d;
				}
			case OP_NEG:								// -INT -> INT
				return new ValueInt(-value);
			case OP_ADD:
				if (ValueInt.TYPENAME.equals(type)) {	// INT + INT -> INT
					return new ValueInt(value + ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT + NUMBER -> NUMBER
					return new ValueNumber(value + ((ValueNumber)t).getValue());
				}
				if (ValueString.TYPENAME.equals(type)) {	// INT + STRING -> STRING
					return new ValueString(value + ((ValueString)t).getValue());
				}
				if (ValueDice.TYPENAME.equals(type)) {	// INT + DICE -> DICE
					ValueDice d=(ValueDice)((ValueDice)t).clone();
					d.addDice(1, value);
					return d;
				}
				break;
			case OP_SUB:
				if (ValueInt.TYPENAME.equals(type)) {	// INT - INT -> INT
					return new ValueInt(value - ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT - NUMBER -> NUMBER
					return new ValueNumber(value - ((ValueNumber)t).getValue());
				}
				break;
			case OP_MUL:
				if (ValueInt.TYPENAME.equals(type)) {	// INT * INT -> INT
					return new ValueInt(value * ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT * NUMBER -> NUMBER
					return new ValueNumber(value * ((ValueNumber)t).getValue());
				}
				break;
			case OP_DIV:
				if (ValueInt.TYPENAME.equals(type)) {	// INT / INT -> INT
					return new ValueInt(value / ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT / NUMBER -> NUMBER
					return new ValueNumber(value / ((ValueNumber)t).getValue());
				}
				break;
			case COMP_EQ:
				if (ValueInt.TYPENAME.equals(type)) {	// INT = INT -> bool
					return new ValueBool(value == ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT = NUMBER -> bool
					return new ValueBool(value == ((ValueNumber)t).getValue());
				}
				break;
			case COMP_NE:
				if (ValueInt.TYPENAME.equals(type)) {	// INT <> INT -> bool
					return new ValueBool(value != ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT <> NUMBER -> bool
					return new ValueBool(value != ((ValueNumber)t).getValue());
				}
				break;
			case COMP_LE:
				if (ValueInt.TYPENAME.equals(type)) {	// INT <= INT -> bool
					return new ValueBool(value <= ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT <= NUMBER -> bool
					return new ValueBool(value <= ((ValueNumber)t).getValue());
				}
				break;
			case COMP_LT:
				if (ValueInt.TYPENAME.equals(type)) {	// INT < INT -> bool
					return new ValueBool(value < ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT < NUMBER -> bool
					return new ValueBool(value < ((ValueNumber)t).getValue());
				}
				break;
			case COMP_GE:
				if (ValueInt.TYPENAME.equals(type)) {	// INT >= INT -> bool
					return new ValueBool(value >= ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT >= NUMBER -> bool
					return new ValueBool(value >= ((ValueNumber)t).getValue());
				}
				break;
			case COMP_GT:
				if (ValueInt.TYPENAME.equals(type)) {	// INT > INT -> bool
					return new ValueBool(value > ((ValueInt)t).getValue());
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT > NUMBER -> bool
					return new ValueBool(value > ((ValueNumber)t).getValue());
				}
				break;
		}
		return super.calculate(op, t);	// let the superclass throw the exception
	}

	/**
	 * For the static type-checking. Return the expected type for an
	 * operation.
	 */
	public String getResultType(int op, String type) {
		switch(op) {
			case TO_INT:	return ValueInt.TYPENAME;
			case TO_NUMBER:	return ValueNumber.TYPENAME;
			case TO_DICE:	return ValueDice.TYPENAME;
			case OP_NEG:	return ValueInt.TYPENAME;	// -INT -> INT
			case OP_ADD:
				if (ValueInt.TYPENAME.equals(type)) {	// INT + INT -> INT
					return ValueInt.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT + NUMBER -> NUMBER
					return ValueNumber.TYPENAME;
				}
				if (ValueString.TYPENAME.equals(type)) {	// INT + STRING -> STRING
					return ValueString.TYPENAME;
				}
				if (ValueDice.TYPENAME.equals(type)) {	// INT + DICE -> DICE
					return ValueDice.TYPENAME;
				}
				break;
			case OP_SUB:
				if (ValueInt.TYPENAME.equals(type)) {	// INT - INT -> INT
					return ValueInt.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT - NUMBER -> NUMBER
					return ValueNumber.TYPENAME;
				}
				break;
			case OP_MUL:
				if (ValueInt.TYPENAME.equals(type)) {	// INT * INT -> INT
					return ValueInt.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT * NUMBER -> NUMBER
					return ValueNumber.TYPENAME;
				}
				break;
			case OP_DIV:
				if (ValueInt.TYPENAME.equals(type)) {	// INT / INT -> INT
					return ValueInt.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT / NUMBER -> NUMBER
					return ValueNumber.TYPENAME;
				}
				break;
			case COMP_EQ:
				if (ValueInt.TYPENAME.equals(type)) {	// INT = INT -> bool
					return ValueBool.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT = NUMBER -> bool
					return ValueBool.TYPENAME;
				}
				break;
			case COMP_NE:
				if (ValueInt.TYPENAME.equals(type)) {	// INT <> INT -> bool
					return ValueBool.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT <> NUMBER -> bool
					return ValueBool.TYPENAME;
				}
				break;
			case COMP_LE:
				if (ValueInt.TYPENAME.equals(type)) {	// INT <= INT -> bool
					return ValueBool.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT <= NUMBER -> bool
					return ValueBool.TYPENAME;
				}
				break;
			case COMP_LT:
				if (ValueInt.TYPENAME.equals(type)) {	// INT < INT -> bool
					return ValueBool.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT < NUMBER -> bool
					return ValueBool.TYPENAME;
				}
				break;
			case COMP_GE:
				if (ValueInt.TYPENAME.equals(type)) {	// INT >= INT -> bool
					return ValueBool.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT >= NUMBER -> bool
					return ValueBool.TYPENAME;
				}
				break;
			case COMP_GT:
				if (ValueInt.TYPENAME.equals(type)) {	// INT > INT -> bool
					return ValueBool.TYPENAME;
				}
				if (ValueNumber.TYPENAME.equals(type)) {	// INT > NUMBER -> bool
					return ValueBool.TYPENAME;
				}
				break;
		}
		return super.getResultType(op, type);	// let the superclass return the error
	}
	
	//----------------------------------------------------------------------
	//------- overridden methods
	//----------------------------------------------------------------------
	/** Create a copy of this object */
	public Object clone() {return new ValueInt(value);}
		
}