package de.mutschler.rpdl.calc;

import de.mutschler.rpdl.*;
import de.mutschler.rpdl.entity.*;

public class CalcAttr extends CalcItem
{
	/** The attribute to calculate. */
	private RpdlAttribute attr;
	
	public CalcAttr(RpdlAttribute attr) {
		this.attr = attr;
	}

	/** Get the associated attribute. */
	public RpdlAttribute getAttribute() {return attr;}
	

	/** execute this calculation. Call the calculation method of the attribute. */
	protected void execute(RpdlBase main) throws RpdlException {
		attr.calculate(main);
	}

	/** This is to specify the order of execution.
	 * returns a negative number if this object should be executed before the passed
	 * object.
	 * return 0 if the order doesn't matter
	 * return positive number, if this object must be executed after the passed object.
	 */
	public int compareTo(Object o) {
		int res = super.compareTo(o);
		if (res != 0) return res;
		if (!(o  instanceof CalcAttr)) return 0;
		CalcAttr ca = (CalcAttr)o;
		try {
			// if the passed attr is in the dependency list of the current attr
			if (attr.getDependencies().contains(ca.getAttribute()))
				return 1;
			// otherwise:
			if (ca.getAttribute().getDependencies().contains(attr))
				return -1;
		} catch (RpdlException re) {
			error("While comparing attribute " + attr.getName() + " with attribute " + ca.getAttribute().getName() + ":", re);
		}
		
		if (getAttribute().equals(ca.getAttribute())) return 0;

		// when no dependency, just pass 0
//		debug("no depency between attr '" + getAttribute() + "' and attr '" + ca.getAttribute());
		return 0;
		
	}


	public boolean equals(Object o) {
		if (o instanceof CalcAttr) 
			return attr.equals(((CalcAttr)o).getAttribute());
		return false;
	}
	
	public String toString() {
		return "Calc Attr '" + attr.getName() + "' of Object '" + attr.getObject().getName() + "'";
	}
}