package de.mutschler.rpdl.calc;

import de.mutschler.util.*;
import de.mutschler.rpdl.*;
import java.util.*;

/**
 * Base class of all events.
 */

public abstract class CalcItem extends LogObj implements Comparable
{
	/** Execute calculation for this object. */
	protected abstract void execute(RpdlBase main) throws RpdlException;
	
	/** This is to specify the order of execution.
	 * returns a negative number if this object should be executed before the passed
	 * object.
	 * return 0 if the order doesn't matter. Note: if this is 0 it does not mean, they are equal!
	 * therefore you can't use this for a sorted list.....
	 * return positive number, if this object must be executed after the passed object.
	 */
	public int compareTo(Object o) {
		CalcItem ci = (CalcItem) o;
		int pri1 = getPriority();
		int pri2 = ci.getPriority();
		if (pri1 < pri2) return -1;
		if (pri1 > pri2) return  1;
/*		
		// assume the objects are of the same type
		if (ci instanceof CalcAttr && this instanceof CalcAttr)
			return ((CalcAttr)this).compareTo((CalcAttr)ci);
*/
		return 0;
	}

	protected static final int PRI_NONE = -1;
	protected static final int PRI_HIGH = 1;
	protected static final int PRI_NORM = 2;
	protected static final int PRI_LOW = 3;
	
	protected int getPriority() {return PRI_NONE;}
}