package de.mutschler.rpdl.command;

import de.mutschler.rpdl.*;
import de.mutschler.rpdl.entity.*;
import de.mutschler.rpdl.basicvalue.*;
import de.mutschler.rpdl.statement.Stmt;
import java.util.*;
import java.text.*;

public class CmdCreateObject extends RpdlCommand implements java.io.Serializable
{
	static final long serialVersionUID = -6712125342910621519L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------

	private String name;
	private Set types;
	private List statements = Collections.EMPTY_LIST;
	/** Flag, is this commands creates a library object. */
	private boolean isLibrary;
	
	//----------------------------------------------------------------------
	//------- Constructor(s)
	//----------------------------------------------------------------------

	/** A create object command.
	 * @param name the name of the object
	 * @param types a list of types for the object
	 */
	public CmdCreateObject(String name, Set types, boolean isLib)
	{
		this.name = name;
		this.types = types;
		this.isLibrary = isLib;
	}

	/** Set the statement-list. */
	public void setStatements(List stmts) {statements = stmts;}
	//----------------------------------------------------------------------
	//------- Overridden methods
	//----------------------------------------------------------------------
	/** Perform the actions when added to the main RPDL object.
	 */
	//----------------------------------------------------------------------
	//------- Overridden methods
	//----------------------------------------------------------------------
	/** Perform the actions when added to the main RPDL object.
	 */
	public void add(RpdlBase main) throws RpdlException
	{
		try {
			// check if we at least one type
			if (types.size() == 0) {
				throw new RpdlException("Object " + name + " has no parent type", getOrigin());
			}
			// get the parenttypes
			Set t = new HashSet();
			for (Iterator it=types.iterator(); it.hasNext(); ) {
				String pname = (String)it.next();
				RpdlType rt = main.getType(pname);
				if (rt == null)
					throw new RpdlException("ParentType " + pname + " doesn't exist for object " + name, getOrigin());
				t.add(rt);
			}
			// create the object
			RpdlObject ro = new RpdlObject(main, name, t);
			ro.setOrigin(getOrigin());
			for (Iterator it = statements.iterator(); it.hasNext(); ) {
				Stmt st = (Stmt)it.next();
				st.checkStatement(main, ro);
				ro.addStatement(st);
			}
			if (isLibrary) {
				main.addLibObject(ro);
			}
			else {
				main.addObject(ro);
			}
		} catch (RpdlException re) {
			re.fillOrigin(getOrigin());	// update origin if necessary
			throw re;
		}
	}

	public String toString() {
		return "CmdCreateObject: isLib=" + isLibrary + ", " + name;
	}
}