package de.mutschler.rpdl.command;

import de.mutschler.rpdl.*;
import de.mutschler.rpdl.entity.*;
import de.mutschler.rpdl.basicvalue.*;
import de.mutschler.rpdl.statement.Stmt;
import java.util.*;
import java.text.*;

public class CmdCreateType extends RpdlCommand implements java.io.Serializable
{
	static final long serialVersionUID = 7087040593080936924L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------

	private String name;
	private Set types;
	private List statements = Collections.EMPTY_LIST;
	
	//----------------------------------------------------------------------
	//------- Constructor(s)
	//----------------------------------------------------------------------

	/** A create object command.
	 * @param name the name of the object
	 * @param types a list of types for the object
	 */
	public CmdCreateType(String name, Set types)
	{
		this.name = name;
		this.types = types;
	}

	/** Set the statement-list. */
	public void setStatements(List stmts) {
		debug("setting " + stmts.size() + " statements for type " + name);
		if (stmts==null)
			statements = new LinkedList();
		else {
			stmts.remove(null);	// remove null-objects
			statements = stmts;
		}
	}
	//----------------------------------------------------------------------
	//------- Overridden methods
	//----------------------------------------------------------------------
	/** Perform the actions when added to the main RPDL object.
	 */
	//----------------------------------------------------------------------
	//------- Overridden methods
	//----------------------------------------------------------------------
	private transient RpdlType rtype = null;
	/** Perform the actions when added to the main RPDL object.
	 */
	public void add(RpdlBase main) throws RpdlException
	{
		try {
			// get the parenttypes
			Set t = new HashSet();
			for (Iterator it=types.iterator(); it.hasNext(); ) {
				String pname = (String)it.next();
				RpdlType rt = main.getType(pname);
				if (rt == null)
					throw new RpdlException("ParentType " + pname + " doesn't exist for type " + name, getOrigin());
				t.add(rt);
			}
			// create the actual type
			RpdlType rtype = new RpdlType(main, name, t);
			rtype.setOrigin(getOrigin());
			// add the statements to the type
			for (Iterator it = statements.iterator(); it.hasNext(); ) {
				Stmt st = (Stmt)it.next();
				st.checkStatement(main, rtype);
				rtype.addStatement(st);
			}
			main.addType(rtype);
		} catch (RpdlException re) {
			re.fillOrigin(getOrigin());	// update origin if necessary
			throw re;
		}
	}

	public String toString() {
		return "CmdCreateType: " + rtype;
	}
}