package de.mutschler.rpdl.entity;

import de.mutschler.util.LogObj;
import de.mutschler.rpdl.basicvalue.*;
import de.mutschler.rpdl.entity.*;
import de.mutschler.rpdl.*;

public class RpdlTableEntry extends LogObj implements Comparable, java.io.Serializable
{
	static final long serialVersionUID = 7571381550965966302L;
	//----------------------------------------------------------------------
	//------- Attributes
	//----------------------------------------------------------------------
	
	/** The starting of the range */
	private BasicValue start;
	/** The functionality of the starting range.
	 * The types possible here are taken from Bysictype.COMP_xx
	 * Note: the relation xx op start.
	 */
	private int startType;
	
	/** The end of the range */
	private BasicValue end;
	/** The functionality of the end range. */
	private int endType;
	
	/** The formula to calculate for this range. */
	private RpdlFormula formula;
	
	//----------------------------------------------------------------------
	//------- Constructor(s)
	//----------------------------------------------------------------------
	
	public RpdlTableEntry(BasicValue start, int stype, BasicValue end, int etype, RpdlFormula fml)
	{
		this.start = start;
		this.startType = stype;
		this.end = end;
		this.endType = etype;
		this.formula = fml;
	}

	/** Get the value of the start of the range. */
	public BasicValue getStartValue() { return start; }
	
	//-------------------
	// Origin handling
	private Origin origin = null;
	/** Get the origin of this command. */
	public Origin getOrigin() {return origin;}
	/** Set the origin of this command. */
	public void setOrigin(Origin o) {origin=o;}

	//----------------------------------------------------------------------
	//------- Methods
	//----------------------------------------------------------------------
	
	/** Get the formula. */
	public RpdlFormula getFormula() {return formula;}

	/** Set a new formula. */
	protected void setFormula(RpdlFormula f) {formula = f;}
	
	/**
	 * check if the value is in the current range.
	 */
	public boolean inRange(BasicValue x) throws RpdlException {
		ValueBool res;
		// check the start of the range
		res = (ValueBool)x.calculate(startType, start);
		// start range is false, then return.
		if (!res.getValue()) return false;
		// check the end
		// if we don't have an end, then return true
		if (end == null) return true;
		// compare to the end
		res = (ValueBool)x.calculate(endType, end);
		return res.getValue();
		
	}
	
	//----------------------------------------------------------------------
	//------- Overridden system methods
	//----------------------------------------------------------------------
	
	public int compareTo(Object o) {
		try {
			RpdlTableEntry tr = (RpdlTableEntry)o;
			ValueBool bt = (ValueBool)start.calculate(BasicValue.COMP_LT, tr.getStartValue());
			if (bt.getValue()) return -1;
			bt = (ValueBool)start.calculate(BasicValue.COMP_EQ, tr.getStartValue());
			if (bt.getValue()) return 0;
			else return 1;
		} catch (RpdlException re) {
			error("failed to compare ranges.", re);
			throw new RuntimeException("failed to compare ranges");
		}
	}
	
	public boolean equals(Object o) {
		try {
			RpdlTableEntry tr = (RpdlTableEntry)o;
			ValueBool bt = (ValueBool)start.calculate(BasicValue.COMP_EQ, tr.getStartValue());
			return bt.getValue();
		} catch (RpdlException re) {
			error("failed to compare ranges.", re);
			throw new RuntimeException("failed to compare ranges");
		}
    }		
	
	public String toString() {
		StringBuffer sb = new StringBuffer(20);
		sb.append("RANGE x");
		sb.append(BasicValue.getOpName(startType));
		sb.append(start.toString());
		if (end != null) {
			sb.append(" && x");
			sb.append(BasicValue.getOpName(endType));
			sb.append(end.toString());
		}
		sb.append(": ");
		sb.append(formula.toString());
		sb.append(';');
		return sb.toString();
	}

}

