package de.mutschler.rpdl.tree;

import de.mutschler.rpdl.*;
import de.mutschler.rpdl.entity.*;
import java.util.*;

/**
 * This class implements the deep scan method.
 * e.g:
 * + 1
 * | + 11
 * | | + 111
 * | + 12
 * + 2
 *   + 21
 *
 * will result in the order 1,11,111,12,2,21
 */

public class DeepOrder extends ListOrder
{
	/** create the sequential list of objects. */
	protected void updateObjectList(List list) {
		debug("creating new treelist");
		list.clear();
		// start with the base object
		addObject(list, getMain().getBaseObject());
	}
	
	private void addObject(List list, RpdlObject obj) {
		// we should not get any null-objects, but who knows...
		if (obj == null) return;
		// ignore this object, if it is already in the list
		if (list.contains(obj))	return;
		// not there, so add it
		list.add(obj);
		// now add the subobjects
		for (Iterator it = obj.getSubObjects(); it.hasNext();) {
			RpdlObject ro = (RpdlObject)it.next();
			// add it recursively (!)
			addObject(list,ro);
		}
	}
}
