package de.mutschler.util;

/**
 * Utility methods for string-handling.
 */
public abstract class StringUtil	// declared abstract, so one can't instantiate it
{
	/**
	 * This method replaces escape-sequences in a string.
	 */
	public static String unEscape(String s) {
		StringBuffer res = new StringBuffer(s.length());
		for (int i=0; i<s.length(); i++) {
			char c = s.charAt(i);
			if (c == '\\') {
				// we have a backslash. Get the next char.
				if (++i<s.length()) {
					c = s.charAt(i);
					switch (c) {
						case 'n': res.append(LogObj.CR); break;
						case 'r': res.append('\r'); break;
						case 't': res.append('\t'); break;
						case 'b': res.append('\b'); break;
						case '\\': res.append('\\'); break;
						default: res.append(c);
					}
					break;
				}
			}
			
			// default:
			res.append(c);
		}
		return res.toString();			
	}
	
	/** Test if a string is empty. */
	public static boolean isEmpty(String s) {
		if (s == null) return true;	// null is an empty string too
		for (int i=0; i<s.length(); i++) {
			if (!Character.isWhitespace(s.charAt(i)))
				return false;
		}
		return true;
	}
	
	/**
	 * Indent a string. Take care of multiple lines in the string!
	 * The mthod is simple: append after each \n the prefix.
	 * @param s the string to parse
	 * @param prefix the prefix for each line
	 */
	public static String indent(String s, String prefix) {
		return indent(s, prefix, prefix);
	}

	/**
	 * Indent a string. Take care of multiple lines in the string!
	 * The mthod is simple: append after each \n the prefix.
	 * @param s the string to parse
	 * @param prefixFirst the prefix for the first line
	 * @param prefixRest the prefix for the other lines
	 */
	public static String indent(String s, String prefixFirst, String prefixRest) {
		StringBuffer sb = new StringBuffer(s.length() + 20);
		sb.append(prefixFirst);
		boolean r = false;	// flag if we found a \r
		for (int i=0; i<s.length(); i++) {
			char c = s.charAt(i);
			switch(c) {
				case '\n': sb.append(LogObj.CR).append(prefixRest); r = false; break;
				case '\r': if (r) sb.append('\r'); r = true; break;
				default:
					if (r) sb.append('\r');
					r = false;
					sb.append(c); break;
			}
		}
		return sb.toString();
	}
	
	/** Indent a string with the default of 2 spaces. */
	public static String indent(String s) {return indent(s, "  ");}
	
	/** Make an Integer of fixed length.
	 * e.g. fmt(4,3) -> "004"
	 */
	public static String fmt(int i, int len) {
		String s = Integer.toString(i);
		if (s.length() > len) {
			Log.error(".StringUtil", "" + i  +" longer than " + len + " chars");
			return s.substring(0, len);
		}
		
		if (s.length() == len) return s;
		
		StringBuffer sb = new StringBuffer(len);
		for (int j=0; j<(len - s.length()); j++)
			sb.append('0');
		sb.append(s);
		return sb.toString();
	}
	
}